#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>


#define LEVEL 100

unsigned int percent[8] = {
	26,    	//10%
	51,   	//20%
	77,   	//30%
	102,   	//40%
	128,   	//50%
	166,   	//65%
	204,   	//80%
	255,  	//100%
};

void init_ADC(void) {
	ADMUX &= 0xE0;					// keep first 3 bits unchanged
	ADMUX |= (1<<REFS0);			// Avcc = 5V
	ADCSRA = 0;
	ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0));	// 16Mhz/128 = 125Khz (ADC clock)
	ADCSRA |= (1<<ADEN);			// turn on ADC
	ADCSRA |= (1<<ADSC);			// initial conversion
}

void init_Timer0(void) {			// timer 0 for 16 ms interrupt
	TCCR0A |= (1 << COM0A1);		// clear OC0A on compare match
	TCCR0A |= (1 << WGM01);			// CTC mode -> TOP=OCR0A
	TCCR0B |= (1 << CS02);			// prescaler clk/256
	TIMSK0 |= (1 << OCIE0A);		// interrupt enable
	
	OCR0A = 255;
}

void init_Timer2(void) {
	TCCR2A |= (1<<COM2A1);			// on compare match : down set OC2A, up clear OC2A
	TCCR2A |= (1<<WGM20);			// PWM phase correct mode ( 0 0 1 ) 
	TCCR2B |= (1<<CS22)|(1<<CS21);	// prescaler clk/256 ; set timer clock
	
	OCR2A = 255; 
}

int get_val_ADC(uint8_t channel) {
	ADMUX &= 0xE0;						// keep first 3 bits unchanged
	ADMUX |= channel;					// mark channel for read
	
	ADCSRA |= (1 << ADSC);				// starts a new conversion
	while (!(ADCSRA  & (1 << ADIF)));	// conversion is done: ADIF=1
	return ADC;							// returns the ADC value of the chosen channel
}


ISR(TIMER0_COMPA_vect) {
	int i;
	for (i = 0; i < 8; i++) {
		int val = get_val_ADC(i);	// get value from current channel
		if (val > LEVEL) {			// if IR beam is interrupted
			OCR2A = percent[i];		// update OCR2A value
			break;
		}
	}
}


int main(void)
{
	DDRD |= (1 << DDD7); 	// set PD7 out to power led
	init_ADC();
	init_Timer0();
	init_Timer2();
	sei();					// enable global interrupts
	OCR2A = 26;
	while (1);				// run forever waiting for interrupts
	return 0;
}
